﻿using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;

namespace David.Academy.InformationTechnology.AdoNetHealthyFood
{
	class Program
	{
		static void Main(string[] args)
		{
			while (true)
			{
				Console.WriteLine("Input command:");
				Console.WriteLine("1. Print producers count");
				Console.WriteLine("2. Convert energy values to kJ");
				Console.WriteLine("3. Convert energy values to kcal");
				Console.WriteLine("4. Print producers list");
				Console.WriteLine("5. Print brands list");
				Console.WriteLine("6. Print brands list for producer");
				Console.WriteLine("7. Input new product");
				Console.WriteLine("99. Exit");
				int choice = 0;
				int.TryParse(Console.ReadLine(), out choice);
				switch (choice)
				{
					case 1:
						PrintProducersCount();
						break;
					case 2:
						ConvertEnergyValuesToKj();
						break;
					case 3:
						ConvertEnergyValuesToKcal();
						break;
					case 4:
						PrintProducers();
						break;
					case 5:
						PrintBrands();
						break;
					case 6:
						PrintProducerBrands();
						break;
					case 7:
						InputNewProduct();
						break;
					case 99:
						return;
				}
				Console.WriteLine();
			}
		}

		private static SqlConnection CreateConnection()
		{
			string connectionString =
				ConfigurationManager.ConnectionStrings["HealthyFood"].ConnectionString;
			SqlConnection connection = new SqlConnection(connectionString);
			connection.Open();
			return connection;
		}

		private static void PrintProducersCount()
		{
			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText = "SELECT COUNT(*) FROM [Producers]";
				int count = (int)command.ExecuteScalar();
				Console.WriteLine("Number of producers: {0}", count);
			}
		}

		private static void ConvertEnergyValuesToKj()
		{
			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText = "UPDATE [Products] SET [EnergyValue] = [EnergyValue]*4.184";
				int rowCount = command.ExecuteNonQuery();
				Console.WriteLine("Number of rows affected: {0}", rowCount);
			}
		}

		private static void ConvertEnergyValuesToKcal()
		{
			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText = "UPDATE [Products] SET [EnergyValue] = [EnergyValue]/4.184";
				int rowCount = command.ExecuteNonQuery();
				Console.WriteLine("Number of rows affected: {0}", rowCount);
			}
		}

		private static void PrintProducers()
		{
			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText = "SELECT [ProducerID] AS [ID], [Name] FROM [Producers]";
				using (SqlDataReader reader = command.ExecuteReader())
				{
					while (reader.Read())
					{
						Console.WriteLine("Producer ID: {0}; Producer name: {1}",
							reader["ID"], reader["Name"]);
					}
				}
			}
		}

		private static void PrintBrands()
		{
			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText = "SELECT [BrandID], [Name], [Description] FROM [Brands]";
				using (SqlDataReader reader = command.ExecuteReader())
				{
					while (reader.Read())
					{
						string description = (reader.IsDBNull(2)
												? "<NULL>"
												: (string)reader["Description"]);
						Console.WriteLine("Brand ID: {0}; Brand name: {1}; Description: {2}",
							reader["BrandID"], reader["Name"], description);
					}
				}
			}
		}

		private static void PrintProducerBrands()
		{
			Console.Write("Input producer ID: ");
			int producerID = int.Parse(Console.ReadLine());

			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandText =
@"SELECT [BrandID], [Name], [Description]
  FROM [Brands]
  WHERE ([ProducerID] = @producer_id)";
				command.Parameters.AddWithValue("@producer_id", producerID);

				using (SqlDataReader reader = command.ExecuteReader())
				{
					while (reader.Read())
					{
						string description = (reader.IsDBNull(2)
												? "<NULL>"
												: (string)reader["Description"]);
						Console.WriteLine("Brand ID: {0}; Brand name: {1}; Description: {2}",
							reader["BrandID"], reader["Name"], description);
					}
				}
			}
		}

		private static void InputNewProduct()
		{
			Console.Write("Input brand ID: ");
			int brandID = int.Parse(Console.ReadLine());
			Console.Write("Input product name: ");
			string name = Console.ReadLine();
			Console.Write("Input energy value: ");
			decimal energyValue = decimal.Parse(Console.ReadLine());
			Console.Write("Input proteins: ");
			string line = Console.ReadLine();
			object proteins = (string.IsNullOrEmpty(line)
								? DBNull.Value
								: (object)decimal.Parse(line));
			Console.Write("Input fats: ");
			line = Console.ReadLine();
			object fats = (string.IsNullOrEmpty(line)
							? DBNull.Value
							: (object)decimal.Parse(line));
			Console.Write("Input carbohydrates: ");
			line = Console.ReadLine();
			object carbohydrates = (string.IsNullOrEmpty(line)
										? DBNull.Value
										: (object)decimal.Parse(line));

			using (SqlConnection connection = CreateConnection())
			using (SqlCommand command = connection.CreateCommand())
			{
				command.CommandType = CommandType.StoredProcedure;
				command.CommandText = "[CreateProduct]";
				command.Parameters.AddWithValue("@vBrandID", brandID);
				command.Parameters.AddWithValue("@vName", name);
				command.Parameters.AddWithValue("@vEnergyValue", energyValue);
				command.Parameters.AddWithValue("@vProteins", proteins);
				command.Parameters.AddWithValue("@vFats", fats);
				command.Parameters.AddWithValue("@vCarbohydrates", carbohydrates);
				command.Parameters.Add("@vProductID", SqlDbType.Int).Direction
					= ParameterDirection.Output;
				command.ExecuteNonQuery();

				Console.WriteLine("Created product ID: {0}",
					(int)command.Parameters["@vProductID"].Value);
			}
		}
	}
}
