﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace David.Academy.InformationTechnology.LinqExamples
{
	class Program
	{
		private static readonly int[] Numbers
			= new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };

		static void Main(string[] args)
		{
			while (true)
			{
				Console.WriteLine("Input command:");
				Console.WriteLine("1. Use extension method");
				Console.WriteLine("2. LINQ to Objects example 1");
				Console.WriteLine("3. LINQ to Objects example 2");
				Console.WriteLine("4. LINQ to Objects example 3");
				Console.WriteLine("5. LINQ to Objects example 4");
				Console.WriteLine("99. Exit");
				int choice = 0;
				int.TryParse(Console.ReadLine(), out choice);
				switch (choice)
				{
					case 1:
						UseExtensionMethod();
						break;
					case 2:
						LinqExample1();
						break;
					case 3:
						LinqExample2();
						break;
					case 4:
						LinqExample3();
						break;
					case 5:
						LinqExample4();
						break;
					case 99:
						return;
				}
				Console.WriteLine();
			}
		}

		private static void UseExtensionMethod()
		{
			Student student = new Student();
			student.Name = "Пешо";
			student.Score = 5.67;

			student.PrintToConsole();
		}

		private static void LinqExample1()
		{
			IEnumerable<int> result =
				from n in Numbers
				select n * 2;

			foreach (int n in result)
			{
				Console.WriteLine(n);
			}
		}

		private static void LinqExample2()
		{
			IEnumerable<int> result =
				from n in Numbers
				where n > 3
				orderby n % 4
				select n;

			foreach (int n in result)
			{
				Console.WriteLine(n);
			}
		}

		private static void LinqExample3()
		{
			IEnumerable<string> result =
				from n in Numbers
				group n by n % 3 into ng
				select string.Format("Group with remainder {0} contains {1} numbers.", ng.Key, ng.Count());

			foreach (string s in result)
			{
				Console.WriteLine(s);
			}
		}

		private static void LinqExample4()
		{
			IEnumerable<Tuple<int, int>> result =
				from n in Numbers
				join m in Numbers on n % 3 equals m % 3
				select Tuple.Create(n, m);

			foreach (Tuple<int, int> tuple in result)
			{
				Console.WriteLine("({0}, {1})", tuple.Item1, tuple.Item2);
			}
		}
	}
}
