﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Serialization;

namespace XmlExamples
{
	class Program
	{
		static void Main(string[] args)
		{
			while (true)
			{
				Console.WriteLine("Input command:");
				Console.WriteLine("1. Create XML document with the XmlDocument class");
				Console.WriteLine("2. Read XML document with the XmlDocument class");
				Console.WriteLine("3. Create XML document with LINQ to XML");
				Console.WriteLine("4. Read XML document with LINQ to XML");
				Console.WriteLine("5. Create XML document with XML serialization");
				Console.WriteLine("6. Read XML document with XML serialization");
				Console.WriteLine("99. Exit");
				int choice = 0;
				int.TryParse(Console.ReadLine(), out choice);
				switch (choice)
				{
					case 1:
						CreateXmlDocumentWithXmlDocument();
						break;
					case 2:
						ReadXmlDocumentWithXmlDocument();
						break;
					case 3:
						CreateXmlDocumentWithLinqToXml();
						break;
					case 4:
						ReadXmlDocumentWithLinqToXml();
						break;
					case 5:
						CreateXmlDocumentWithXmlSerializer();
						break;
					case 6:
						ReadXmlDocumentWithXmlSerializer();
						break;
					case 99:
						return;
				}
				Console.WriteLine();
			}
		}

		private static void CreateXmlDocumentWithXmlDocument()
		{
			XmlDocument xmlDocument = new XmlDocument();

			XmlDeclaration xmlDeclaration =
				xmlDocument.CreateXmlDeclaration("1.0", "utf-8", "yes");
			xmlDocument.AppendChild(xmlDeclaration);

			XmlElement libraryElement = xmlDocument.CreateElement("library");
			xmlDocument.AppendChild(libraryElement);

			XmlElement booksElement = xmlDocument.CreateElement("books");
			libraryElement.AppendChild(booksElement);

			XmlElement bookElement = xmlDocument.CreateElement("book");
			booksElement.AppendChild(bookElement);

			XmlAttribute authorAttribute = xmlDocument.CreateAttribute("author");
			authorAttribute.Value = "J. R. R. Tolkien";
			bookElement.Attributes.Append(authorAttribute);

			XmlElement titleElement = xmlDocument.CreateElement("title");
			bookElement.AppendChild(titleElement);

			XmlText titleText = xmlDocument.CreateTextNode("The Two Towers");
			titleElement.AppendChild(titleText);

			xmlDocument.Save(@"C:\library.xml");
		}

		private static void ReadXmlDocumentWithXmlDocument()
		{
			XmlDocument xmlDocument = new XmlDocument();
			xmlDocument.Load(@"C:\library.xml");
			XmlElement libraryElement = xmlDocument["library"];
			XmlElement booksElement = libraryElement["books"];
			XmlNodeList bookElements = booksElement.GetElementsByTagName("book");

			foreach (XmlElement bookElement in bookElements)
			{
				XmlAttribute authorAttribute = bookElement.Attributes["author"];
				XmlElement titleElement = bookElement["title"];
				Console.WriteLine("Title: {0}; Author: {1}",
					titleElement.InnerText, authorAttribute.Value);
			}
		}

		private static void CreateXmlDocumentWithLinqToXml()
		{
			XAttribute authorAttribute = new XAttribute("author", "J. R. R. Tolkien");
			XElement titleElement = new XElement("title", "The Two Towers");
			XElement bookElement = new XElement("book", authorAttribute, titleElement);
			XElement booksElement = new XElement("books", bookElement);
			XElement libraryElement = new XElement("library", booksElement);
			XDeclaration xmlDeclaration = new XDeclaration("1.0", "utf-8", "yes");
			XDocument xmlDocument = new XDocument(xmlDeclaration, libraryElement);
			xmlDocument.Save(@"C:\library2.xml");
		}

		private static void ReadXmlDocumentWithLinqToXml()
		{
			XDocument xmlDocument = XDocument.Load(@"C:\library2.xml");
			var books =
				from e in xmlDocument
					.Element("library")
					.Element("books")
					.Elements("book")
				where e.Attribute("author") != null
				select new
				{
					Title = e.Element("title").Value,
					Author = e.Attribute("author").Value,
				};

			foreach (var book in books)
			{
				Console.WriteLine("Title: {0}; Author: {1}",
					book.Title, book.Author);
			}
		}

		private static void CreateXmlDocumentWithXmlSerializer()
		{
			Book book = new Book
			{
				Author = "J. R. R. Tolkien",
				Title = "The Two Towers",
			};
			Library library = new Library
			{
				Books = new List<Book> { book },
			};

			var xmlSerializer = new XmlSerializer(typeof(Library));
			using (FileStream stream = File.Create(@"C:\library3.xml"))
			{
				xmlSerializer.Serialize(stream, library);
			}
		}

		private static void ReadXmlDocumentWithXmlSerializer()
		{
			Library library;
			var xmlSerializer = new XmlSerializer(typeof(Library));
			using (FileStream stream = File.OpenRead(@"C:\library3.xml"))
			{
				library = (Library)xmlSerializer.Deserialize(stream);
			}

			foreach (Book book in library.Books)
			{
				Console.WriteLine("Title: {0}; Author: {1}",
					book.Title, book.Author);
			}
		}
	}
}
